# 1. DoS Incorrect Handling of Unexpected HCI ACL_MTU in Bluetooth Host Layer
- Bug Description: During hci Host Stack initialization, the invalid ACL_MTU packets are not properly handled
- Bug Result: In the erroneous code path, a semaphore is not properly initialized, leading to a NULL pointer dereference
- Bug Impact: DoS caused by a bluetooth Controller sending a malicious ACL_MTU packet via HCI

## Bug Details
- Affected code: HCI initialization process handling of buffer size answer, starting in subsys/bluetooth/host/hci_core.c#le_read_buffer_size_complete
- Required Configuration:
	- unset CONFIG_BT_BREDR
	- enable Host-only build (https://docs.zephyrproject.org/latest/guides/bluetooth/bluetooth-arch.html)

High-Level reasoning for bug occurrence:
1. During HCI Host Stack initialization, different parts of configurations are performed
2. For all messages exchanged that way, it is verified that the commands have successfully been transmitted
3. For the message reading the ACL buffer size, it is also checked that the command has been transmitted
4. What is not checked, however, is if the contents were of the packet were properly processed
5. The situation where the handling fails arrises if the HCI Controller indicates an ACL MTU of 0
6. In this case, the initialization of a semaphore will silently be omitted, leaving it in an uninitialized state
7. As the initialization does not check for this error condition, the bluetooth stack will later use it without initialization, leading to a crash of the bluetooth stack

Vulnerable code path:
1. hci_init->le_init
	- During HCI Host Initialiazation, different hci commands are sent and error conditions are checked 
	- Link: https://github.com/zephyrproject-rtos/zephyr/blob/d969aced6dd9ef7834d21ff8131aa561e71cc42d/subsys/bluetooth/host/hci_core.c#L5602
	- Among other things, the Buffer size is initialized between the host- and the controller stack.
	- Link: https://github.com/zephyrproject-rtos/zephyr/blob/d969aced6dd9ef7834d21ff8131aa561e71cc42d/subsys/bluetooth/host/hci_core.c#L5629
	- It is checked that the command itself is transmitted successfully
	- Link: https://github.com/zephyrproject-rtos/zephyr/blob/d969aced6dd9ef7834d21ff8131aa561e71cc42d/subsys/bluetooth/host/hci_core.c#L5631
2. hci_init->le_init->le_read_buffer_size_complete
	- In case the ACL_MTU is set to zero, the completion function is returned early
	- Link: https://github.com/zephyrproject-rtos/zephyr/blob/d969aced6dd9ef7834d21ff8131aa561e71cc42d/subsys/bluetooth/host/hci_core.c#L5318
	- In this error case, the ACL packets semaphore is not initialized
	- Link: https://github.com/zephyrproject-rtos/zephyr/blob/d969aced6dd9ef7834d21ff8131aa561e71cc42d/subsys/bluetooth/host/hci_core.c#L5325
3. hci_init->le_init
	- The status of parsing the actual ACL_MTU value is not actually checked
	- Link: https://github.com/zephyrproject-rtos/zephyr/blob/d969aced6dd9ef7834d21ff8131aa561e71cc42d/subsys/bluetooth/host/hci_core.c#L5634
4. send_buf->send_frag
	- As the BT stack may go on with its initialization, the uninitialized semaphore will still get used
	- When trying to send a packet, the BT stack will then try to take the uninitialized semaphore, which eventually leads to a crash in the semaphore-related kernel function `k_sem_take`->...->`z_reset_time_slice`
	- Link: https://github.com/zephyrproject-rtos/zephyr/blob/d969aced6dd9ef7834d21ff8131aa561e71cc42d/subsys/bluetooth/host/conn.c#L1132
	

## Proposed Fix
- Add error handling of the ACL buffer size handshake during HCI initialization
    - Link: https://github.com/zephyrproject-rtos/zephyr/blob/d969aced6dd9ef7834d21ff8131aa561e71cc42d/subsys/bluetooth/host/hci_core.c#L5634
